import logging
from flask import request
from flask_jwt_extended import create_access_token, create_refresh_token, get_jwt_identity, \
    jwt_refresh_token_required, jwt_required
from flask_restplus import Resource, Namespace

from portal import jwt
from portal.service import general
from portal.service.auth import login_component, login_user
from portal.service.errors import PortalAPIError, UnauthorizedError
from portal.tools.decorators import error_handler

log = logging.getLogger(__name__)

auth_namespace = Namespace('auth')


@jwt.user_claims_loader
def add_claims_to_access_token(identity):
    data = 'user'
    if general.find_component(identity, throws=False):
        data = 'component'
    return {
        'type': data
        }


# basic login - username + password (user) / name + secret (component)
@auth_namespace.route('/login')
class Login(Resource):

    def post(self):
        data = request.get_json()
        if not data.get('type'):
            raise PortalAPIError(400, message="Missing login type.")

        if data['type'] == 'user':
            username = data.get('username', None)
            password = data.get('password', None)
            gitlab_access_token = data.get('oauth_token', None)

            client = login_user(gitlab_access_token, password, username)
        elif data['type'] == 'component':
            name = data.get('name')
            if not name:
                raise PortalAPIError(400, message="Missing component name.")

            secret = data.get('secret')
            if not secret:
                raise PortalAPIError(400, message="Missing component secret.")

            sender_ip_address = request.remote_addr
            client = login_component(name, sender_ip_address, secret)
        else:
            raise PortalAPIError(400, message="Invalid login type.")

        response = dict(
            id=client.id,
            access_token=create_access_token(identity=client.id),
            refresh_token=create_refresh_token(identity=client.id)
            )
        return response, 200


@auth_namespace.route('/refresh')
class Refresh(Resource):

    @jwt_refresh_token_required
    def post(self):
        client = get_jwt_identity()
        if not client:
            raise UnauthorizedError()

        ret = dict(
            access_token=create_access_token(identity=client)
            )

        return ret, 200


@auth_namespace.route('/logout')
class Logout(Resource):

    @jwt_required
    def post(self):
        # might use redis later
        current_user = get_jwt_identity()

        if not current_user:
            raise UnauthorizedError()

        ret = {
            'access_token': None,
            'refresh_token': None,
            }

        return ret, 200
