import logging
from flask_jwt_extended import jwt_required
from flask_restplus import Resource, Namespace

from portal.rest import rest_helpers
from portal.rest.schemas import component_schema, components_schema
from portal.service import auth, permissions
from portal.service.auth import find_client
from portal.service.components import create_component, delete_component, find_all_components, \
    update_component
from portal.service.errors import ForbiddenError
from portal.service.general import find_component
from portal.tools.decorators import error_handler

components_namespace = Namespace('components')
log = logging.getLogger(__name__)


@components_namespace.route('')
class ComponentList(Resource):
    @jwt_required
    def get(self):
        client = auth.find_client()
        # authorization
        if not permissions.check_sysadmin(client):
            raise ForbiddenError(uid=client.id)

        components_list = find_all_components()
        return components_schema.dump(components_list)[0], 200


    @jwt_required
    def post(self):
        client = auth.find_client()
        # authorization
        if not permissions.check_sysadmin(client):
            raise ForbiddenError(uid=client.id)

        data = rest_helpers.parse_request_data(
            component_schema, action='create', resource='component'
            )
        new_component = create_component(**data)
        return component_schema.dump(new_component)[0], 201


@components_namespace.route('/<string:cid>')
@components_namespace.doc(params={'cid': 'An Component ID'})
class ComponentResource(Resource):

    @jwt_required
    def get(self, cid: str):
        client = find_client()
        if not permissions.check_sysadmin(client):
            raise ForbiddenError(uid=client.id)

        component = find_component(cid)
        return component_schema.dump(component)[0], 200


    @jwt_required
    def delete(self, cid: str):
        client = find_client()
        # authorization
        if not permissions.check_sysadmin(client):
            raise ForbiddenError(uid=client.id)

        component = find_component(cid)
        delete_component(component)
        return '', 204


    @jwt_required
    def put(self, cid: str):
        client = find_client()
        # authorization
        if not permissions.check_sysadmin(client):
            raise ForbiddenError(uid=client.id)

        data = rest_helpers.parse_request_data(
            component_schema, action='update', resource='component'
            )

        component = find_component(cid)
        update_component(component, data)
        return '', 204
