import json
import logging
from flask import jsonify, request
from flask_jwt_extended import jwt_required
from flask_restplus import Namespace, Resource

from portal.database.models import ProjectState
from portal.rest import rest_helpers
from portal.rest.schemas import config_schema, config_schema_reduced, project_schema, \
    projects_schema, submission_create_schema, submission_schema, submissions_schema
from portal.service import auth, general
from portal.service.errors import ForbiddenError, SubmissionRefusedError
from portal.service.permissions import check_client
from portal.service.projects import can_create_submission, create_project, delete_project, \
    find_project_submissions, list_projects, update_project, update_project_config
from portal.service.submissions import create_submission
from portal.tools import time
from portal.tools.decorators import error_handler

projects_namespace = Namespace('')

log = logging.getLogger(__name__)


@projects_namespace.route('/courses/<string:cid>/projects/<string:pid>')
@projects_namespace.doc({'cid': 'Course id', 'pid': 'Project id'})
class ProjectResource(Resource):

    @jwt_required
    def get(self, cid: str, pid: str):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        permissions = ['view_course_full', 'view_course_limited']
        if not (check_client(client=client, course=course, permissions=permissions)):
            raise ForbiddenError(uid=client.id)

        project = general.find_project(course, pid)
        return project_schema.dump(project)


    @jwt_required
    def delete(self, cid: str, pid: str):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        if not check_client(client=client, course=course, permissions=['update_course']):
            raise ForbiddenError(uid=client.id)

        project = general.find_project(course, pid)
        delete_project(project)
        return '', 204


    @jwt_required
    def put(self, cid: str, pid: str):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        permissions = ['write_projects', 'update_course']
        if not check_client(client=client, course=course, permissions=permissions):
            raise ForbiddenError(uid=client.id)

        data = rest_helpers.parse_request_data(
            project_schema, action='update', resource='project'
            )

        project = general.find_project(course, pid)
        update_project(project, data)
        return '', 204


@projects_namespace.route('/courses/<string:cid>/projects')
@projects_namespace.doc({'cid': 'Course id'})
class ProjectsList(Resource):

    @jwt_required
    def get(self, cid: str):
        client = auth.find_client()
        course = general.find_course(cid)
        return projects_schema.dump(list_projects(course, client))


    @jwt_required
    def post(self, cid: str):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        if not check_client(client=client, course=course, permissions=['update_course']):
            raise ForbiddenError(uid=client.id)

        data = rest_helpers.parse_request_data(
            project_schema, action='create', resource='project'
            )

        new_project = create_project(course, data)
        return project_schema.dump(new_project)[0], 201


@projects_namespace.route('/courses/<string:cid>/projects/<string:pid>/config')
@projects_namespace.doc({'cid': 'Course id', 'pid': 'Project id'})
class ProjectConfigResource(Resource):

    @jwt_required
    def get(self, cid: str, pid: str):
        client = auth.find_client()
        course = general.find_course(cid)
        project = general.find_project(course, pid)
        # authorization
        if check_client(client=client, course=course, permissions=['view_course_full']):
            return config_schema.dump(project.config)
        elif check_client(client=client, course=course, permissions=['view_course_limited']):
            return config_schema_reduced.dump(project.config)
        else:
            raise ForbiddenError(uid=client.id)


    @jwt_required
    def put(self, cid: str, pid: str):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        permissions = ['write_projects', 'update_course']
        if not check_client(client=client, course=course, permissions=permissions):
            raise ForbiddenError(uid=client.id)

        data = rest_helpers.parse_request_data(
            config_schema, action='update', resource='project_config'
            )

        project = general.find_project(course, pid)
        update_project_config(project, data)
        return '', 204


@projects_namespace.route('/courses/<string:cid>/projects/<string:pid>/submissions')
@projects_namespace.doc({'cid': 'Course id', 'pid': 'Project id'})
class ProjectSubmissions(Resource):

    @jwt_required
    def get(self, cid: str, pid: str):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        if not check_client(client=client, course=course, permissions=['read_submissions_all']):
            raise ForbiddenError(uid=client.id)

        user_id = request.args.get('user')
        project = general.find_project(course, pid)
        submissions = find_project_submissions(project, user_id)
        return submissions_schema.dump(submissions)


    @jwt_required
    def post(self, cid: str, pid: str):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        if not check_client(client=client, course=course, permissions=['create_submissions']):
            raise ForbiddenError(uid=client.id)

        # check if a new submission can be created by this user in this project
        project = general.find_project(course, pid)
        if project.state(timestamp=time.NOW()) != ProjectState.ACTIVE:
            raise SubmissionRefusedError(f"Project {project.name} not active.")
        if not can_create_submission(client, project):
            raise SubmissionRefusedError(f"Submission in project {project.name} already active.")

        data = rest_helpers.parse_request_data(
            submission_create_schema, action='create', resource='submission'
            )

        # data for Kontr processing
        project_params = data.get('project_params')
        project_params_string = json.dumps(project_params) or ""

        new_submission = create_submission(
            user=client,
            project=project,
            file_params=data['file_params'],
            project_params_string=project_params_string
            )
        return submission_schema.dump(new_submission)[0], 201


@projects_namespace.route('/courses/<string:cid>/projects/<string:pid>/files')
@projects_namespace.doc({'cid': 'Course id', 'pid': 'Project id'})
class ProjectTestFiles(Resource):

    def get(self, cid: str, pid: str):
        # Gets test files (feature)
        return jsonify({"message": "Not implemented."})
