import logging
from flask_jwt_extended import jwt_required
from flask_restplus import Resource, Namespace

from portal.rest import rest_helpers
from portal.rest.schemas import permissions_schema, role_schema, roles_schema, \
    user_list_update_schema, users_schema
from portal.service import auth, general
from portal.service.errors import ForbiddenError
from portal.service.permissions import check_client
from portal.service.roles import add_single_user_to_role, create_role, delete_role, list_roles, \
    remove_single_user_from_role, update_role, update_role_permissions, \
    update_users_membership_in_role
from portal.tools.decorators import error_handler

roles_namespace = Namespace('')

log = logging.getLogger(__name__)


@roles_namespace.route('/courses/<string:cid>/roles/<string:rid>')
@roles_namespace.doc({'cid': 'Course id', 'rid': 'Role id'})
class RoleResource(Resource):

    @jwt_required
    def get(self, cid: str, rid: str):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        permissions = ['view_course_full', 'view_course_limited']
        if not (check_client(client=client, course=course, permissions=permissions)):
            raise ForbiddenError(uid=client.id)

        role = general.find_role(course, rid)
        return role_schema.dump(role)


    @jwt_required
    def delete(self, cid, rid):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        if not check_client(client=client, course=course, permissions=['update_course']):
            raise ForbiddenError(uid=client.id)

        role = general.find_role(course, rid)
        delete_role(role)
        return '', 204


    @jwt_required
    def put(self, cid, rid):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        if not check_client(client=client, course=course,
                            permissions=['write_roles', 'update_course']):
            raise ForbiddenError(uid=client.id)

        data = rest_helpers.parse_request_data(
            role_schema, action='update', resource='role'
            )

        role = general.find_role(course, rid)
        update_role(role, data)
        return '', 204


@roles_namespace.route('/courses/<string:cid>/roles')
@roles_namespace.doc({'cid': 'Course id'})
class RoleList(Resource):

    @jwt_required
    def get(self, cid):
        client = auth.find_client()
        course = general.find_course(cid)
        return roles_schema.dump(list_roles(course, client))


    @jwt_required
    def post(self, cid):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        if not check_client(client=client, course=course, permissions=['update_course']):
            raise ForbiddenError(uid=client.id)

        data = rest_helpers.parse_request_data(
            role_schema, action='create', resource='role'
            )
        new_role = create_role(course, **data)
        return role_schema.dump(new_role)[0], 201


@roles_namespace.route('/courses/<string:cid>/roles/<string:rid>/permissions')
@roles_namespace.doc({'cid': 'Course id', 'rid': 'Role id'})
class RolePermissions(Resource):

    @jwt_required
    def get(self, cid, rid):
        client = auth.find_client()
        course = general.find_course(cid)
        role = general.find_role(course, rid)

        # authorization
        if not check_client(client=client, course=course, permissions=['view_course_full']) or \
                (check_client(client=client, course=course, permissions=['view_course_limited'])
                 and client in role.users):
            raise ForbiddenError(uid=client.id)

        return permissions_schema.dump(role.permissions)


    @jwt_required
    def put(self, cid, rid):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        permissions = ['write_roles', 'update_course']
        if not check_client(client=client, course=course, permissions=permissions):
            raise ForbiddenError(uid=client.id)

        data = rest_helpers.parse_request_data(
            permissions_schema, action='update', resource='role_permissions'
            )

        role = general.find_role(course, rid)
        update_role_permissions(role, data)
        return permissions_schema.dump(role.permissions), 204


@roles_namespace.route('/courses/<string:cid>/roles/<string:rid>/users')
@roles_namespace.doc({'cid': 'Course id', 'rid': 'Role id'})
class RoleUsersList(Resource):

    @jwt_required
    def get(self, cid, rid):
        client = auth.find_client()
        course = general.find_course(cid)
        role = general.find_role(course, rid)
        # authorization

        if not check_client(client=client, course=course, permissions=['view_course_full']) or \
                (check_client(client=client, course=course,
                              permissions=['view_course_limited']) and client in role.users):
            raise ForbiddenError(uid=client.id)

        return users_schema.dump(role.users)


    @jwt_required
    def put(self, cid, rid):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        permissions = ['write_roles', 'update_course']
        if not check_client(client=client, course=course, permissions=permissions):
            raise ForbiddenError(uid=client.id)

        data = rest_helpers.parse_request_data(
            user_list_update_schema, action='update', resource='role_membership'
            )

        # everything from users_add is added, THEN everything from users_remove is subtracted
        role = general.find_role(course, rid)
        update_users_membership_in_role(role, data)
        return '', 204


@roles_namespace.route('/courses/<string:cid>/roles/<string:rid>/users/<string:uid>')
@roles_namespace.doc({'cid': 'Course id', 'rid': 'Role id', 'uid': 'User id'})
class RoleUser(Resource):

    @jwt_required
    def put(self, cid, rid, uid):
        client = auth.find_client()
        course = general.find_course(cid)

        # authorization
        permissions = ['write_roles', 'update_course']
        if not check_client(client=client, course=course, permissions=permissions):
            raise ForbiddenError(uid=client.id)

        role = general.find_role(course, rid)
        user = general.find_user(uid)
        add_single_user_to_role(role, user)
        return '', 204


    @jwt_required
    def delete(self, cid, rid, uid):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        permissions = ['write_roles', 'update_course']
        if not check_client(client=client, course=course, permissions=permissions):
            raise ForbiddenError(uid=client.id)

        role = general.find_role(course, rid)
        user = general.find_user(uid)
        remove_single_user_from_role(role, user)
        return '', 204
